@@ -1,3 +1,5 @@
+import os
+import time
 from types import TracebackType
 from typing import Any, List, Literal, Mapping, Optional, Type, overload
 
@@ -7,65 +9,66 @@ from gradientai._base_model import BaseModel, CapabilityFilterOption
 from gradientai._embeddings_model import EmbeddingsModel
 from gradientai._model import Model
 from gradientai._model_adapter import ModelAdapter
+from gradientai._rag import RAGCollection, RAGFile
 from gradientai._types import (
     AnalyzeSentimentParamsExample,
     AnalyzeSentimentResponse,
     AnswerParamsSource,
     AnswerResponse,
     ExtractParamsSchemaValue,
+    ExtractPdfResponse,
     ExtractResponse,
     PersonalizeResponse,
     Sentiment,
     SummarizeParamsExample,
     SummarizeParamsLength,
     SummarizeResponse,
+    TranscribeAudioResponse,
 )
 from gradientai.helpers.env_manager import ENV_MANAGER
 from gradientai.openapi.client.api.blocks_api import BlocksApi
 from gradientai.openapi.client.api.embeddings_api import EmbeddingsApi
+from gradientai.openapi.client.api.files_api import FilesApi
 from gradientai.openapi.client.api.models_api import ModelsApi
+from gradientai.openapi.client.api.rag_api import RAGApi
 from gradientai.openapi.client.api_client import ApiClient
 from gradientai.openapi.client.configuration import Configuration
 from gradientai.openapi.client.models.analyze_sentiment_body_params import (
     AnalyzeSentimentBodyParams,
 )
+from gradientai.openapi.client.models.create_audio_transcription_body_params import (
+    CreateAudioTranscriptionBodyParams,
+)
+from gradientai.openapi.client.models.create_rag_collection_body_params import (
+    CreateRagCollectionBodyParams,
+)
+from gradientai.openapi.client.models.create_rag_collection_body_params_files_inner import (
+    CreateRagCollectionBodyParamsFilesInner,
+)
 from gradientai.openapi.client.models.extract_entity_body_params import (
     ExtractEntityBodyParams,
 )
-from gradientai.openapi.client.models.extract_entity_success import (
-    ExtractEntitySuccess,
-)
 from gradientai.openapi.client.models.generate_answer_body_params import (
     GenerateAnswerBodyParams,
 )
-from gradientai.openapi.client.models.generate_answer_body_params_source import (
-    GenerateAnswerBodyParamsSource,
-)
-from gradientai.openapi.client.models.generate_answer_success import (
-    GenerateAnswerSuccess,
-)
 from gradientai.openapi.client.models.model_adapter import (
     ModelAdapter as ApiModelAdapter,
 )
 from gradientai.openapi.client.models.personalize_document_body_params import (
     PersonalizeDocumentBodyParams,
 )
-from gradientai.openapi.client.models.personalize_document_success import (
-    PersonalizeDocumentSuccess,
-)
 from gradientai.openapi.client.models.summarize_document_body_params import (
     SummarizeDocumentBodyParams,
 )
-from gradientai.openapi.client.models.summarize_document_success import (
-    SummarizeDocumentSuccess,
-)
 
 
 class Gradient:
     _api_client: ApiClient
     _blocks_api: BlocksApi
     _embeddings_api: EmbeddingsApi
+    _files_api: FilesApi
     _models_api: ModelsApi
+    _rag_api: RAGApi
     _workspace_id: str
 
     def __init__(
@@ -112,7 +115,9 @@ class Gradient:
         )
         self._blocks_api = BlocksApi(self._api_client)
         self._embeddings_api = EmbeddingsApi(self._api_client)
+        self._files_api = FilesApi(self._api_client)
         self._models_api = ModelsApi(self._api_client)
+        self._rag_api = RAGApi(self._api_client)
         self._workspace_id = workspace_id
 
     def __enter__(self) -> Self:
@@ -157,8 +162,7 @@ class Gradient:
         *,
         only_base: Literal[True],
         capability: Optional[CapabilityFilterOption],
-    ) -> List[BaseModel]:
-        ...
+    ) -> List[BaseModel]: ...
 
     @overload
     def list_models(
@@ -166,8 +170,7 @@ class Gradient:
         *,
         only_base: Literal[False],
         capability: Optional[CapabilityFilterOption],
-    ) -> List[Model]:
-        ...
+    ) -> List[Model]: ...
 
     def list_models(
         self,
@@ -254,7 +257,9 @@ class Gradient:
             ),
         )
 
-        return AnswerResponse(answer=response.answer)
+        return AnswerResponse(
+            answer=response.answer, rag_context=response.rag_context
+        )
 
     def summarize(
         self,
@@ -267,7 +272,7 @@ class Gradient:
             x_gradient_workspace_id=self._workspace_id,
             summarize_document_body_params=SummarizeDocumentBodyParams(
                 document=document,
-                examples=[] if examples is None else examples,
+                examples=examples,
                 length=None if length is None else length.value,
             ),
         )
@@ -275,15 +280,22 @@ class Gradient:
         return SummarizeResponse(summary=response.summary)
 
     def analyze_sentiment(
-        self, *, document: str, examples: List[AnalyzeSentimentParamsExample]
+        self,
+        *,
+        document: str,
+        examples: Optional[List[AnalyzeSentimentParamsExample]] = None,
     ) -> AnalyzeSentimentResponse:
-        parsed_examples = [
-            {
-                "document": example["document"],
-                "sentiment": example["sentiment"].value,
-            }
-            for example in examples
-        ]
+        parsed_examples = (
+            None
+            if not examples
+            else [
+                {
+                    "document": example["document"],
+                    "sentiment": example["sentiment"].value,
+                }
+                for example in examples
+            ]
+        )
 
         result = self._blocks_api.analyze_sentiment(
             x_gradient_workspace_id=self._workspace_id,
@@ -334,3 +346,177 @@ class Gradient:
             key: value.actual_instance for key, value in result.entity.items()
         }
         return ExtractResponse(entity=entity)
+
+    def extract_pdf(
+        self,
+        *,
+        filepath: str,
+    ) -> ExtractPdfResponse:
+        result = self._blocks_api.extract_pdf(
+            x_gradient_workspace_id=self._workspace_id,
+            file=filepath,
+        )
+
+        pages = [
+            {
+                "images": [
+                    {
+                        "data": image.data,
+                        "format": image.format,
+                    }
+                    for image in page.images
+                ],
+                "page_number": page.page_number,
+                "tables": [
+                    {
+                        "name": table.name,
+                        "table_rows": [
+                            {
+                                "cells": [
+                                    {
+                                        "cell_value": cell.cell_value,
+                                        "col_span": cell.col_span,
+                                        "row_span": cell.row_span,
+                                    }
+                                    for cell in row.cells
+                                ],
+                                "type": row.type,
+                            }
+                            for row in table.table_rows
+                        ],
+                    }
+                    for table in page.tables
+                ],
+                "text": page.text,
+                "text_blocks": [
+                    {
+                        "kind": text_block.kind,
+                        "texts": text_block.texts,
+                    }
+                    for text_block in page.text_blocks
+                ],
+            }
+            for page in result.pages
+        ]
+
+        return ExtractPdfResponse(
+            pages=pages,
+            text=result.text,
+            title=result.title,
+        )
+
+    def transcribe_audio(
+        self,
+        *,
+        filepath: str,
+    ) -> TranscribeAudioResponse:
+        file_result = self._files_api.upload_file(
+            file=filepath,
+            type="audioFile",
+            x_gradient_workspace_id=self._workspace_id,
+        )
+
+        create_transcription_result = self._blocks_api.create_audio_transcription(
+            create_audio_transcription_body_params=CreateAudioTranscriptionBodyParams(
+                file_id=file_result.id,
+            ),
+            x_gradient_workspace_id=self._workspace_id,
+        )
+
+        while True:
+            get_transcription_result = self._blocks_api.get_audio_transcription(
+                transcription_id=create_transcription_result.transcription_id,
+                x_gradient_workspace_id=self._workspace_id,
+            )
+
+            status = get_transcription_result.actual_instance.status
+            if (
+                status == "pending"
+                or status == "pendingCancellation"
+                or status == "running"
+            ):
+                time.sleep(1)
+                continue
+
+            if status == "cancelled" or status == "failed":
+                raise Exception("Unable to get transcription")
+
+            if status == "succeeded":
+                return TranscribeAudioResponse(
+                    text=get_transcription_result.actual_instance.result.text
+                )
+
+            raise Exception(f"Received unexpected status: {status}")
+
+    def create_rag_collection(
+        self,
+        *,
+        name: str,
+        slug: str,
+        filepaths: Optional[List[str]] = None,
+    ) -> RAGCollection:
+        if filepaths is None:
+            filepaths = []
+
+        file_results = [
+            self._files_api.upload_file(
+                file=file_,
+                type="ragUserFile",
+                x_gradient_workspace_id=self._workspace_id,
+            )
+            for file_ in filepaths
+        ]
+
+        rag_result = self._rag_api.create_rag_collection(
+            x_gradient_workspace_id=self._workspace_id,
+            create_rag_collection_body_params=CreateRagCollectionBodyParams(
+                name=name,
+                slug=slug,
+                files=[
+                    CreateRagCollectionBodyParamsFilesInner(
+                        id=file_result.id, name=os.path.basename(file_path)
+                    )
+                    for (file_result, file_path) in zip(file_results, filepaths)
+                ],
+            ),
+        )
+
+        return self.get_rag_collection(id_=rag_result.id)
+
+    def list_rag_collections(self) -> List[RAGCollection]:
+        """Files are not present in the list call. To retrieve the files use `getRagCollection`."""
+        result = self._rag_api.list_rag_collections(
+            x_gradient_workspace_id=self._workspace_id,
+        )
+
+        return [
+            RAGCollection(
+                files_api=self._files_api,
+                files=[],
+                id_=rag_collection.id,
+                name=rag_collection.name,
+                rag_api=self._rag_api,
+                workspace_id=self._workspace_id,
+            )
+            for rag_collection in result.rag_collections
+        ]
+
+    def get_rag_collection(self, *, id_: str) -> RAGCollection:
+        result = self._rag_api.get_rag_collection(
+            id=id_,
+            x_gradient_workspace_id=self._workspace_id,
+        )
+
+        files: List[RAGFile] = [
+            {"id_": file.id, "name": file.name, "status": file.status}
+            for file in result.files
+        ]
+
+        return RAGCollection(
+            files_api=self._files_api,
+            files=files,
+            id_=id_,
+            name=result.name,
+            rag_api=self._rag_api,
+            workspace_id=self._workspace_id,
+        )
