@@ -3,5 +3,7 @@
 # import apis into api package
 from gradientai.openapi.client.api.blocks_api import BlocksApi
 from gradientai.openapi.client.api.embeddings_api import EmbeddingsApi
+from gradientai.openapi.client.api.files_api import FilesApi
 from gradientai.openapi.client.api.models_api import ModelsApi
+from gradientai.openapi.client.api.rag_api import RAGApi
 
